package com.tzld.piaoquan.recommend.sort.service.impl;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.piaoquan.recommend.sort.service.ExampleService;
import com.tzld.piaoquan.recommend.sort.util.HttpClientUtil;
import com.tzld.piaoquan.recommend.sort.util.HttpPoolClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.tzld.piaoquan.recommend.sort.model.vo.ExampleVO;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.util.List;
import java.util.concurrent.*;

@Service
public class ExampleServiceImpl implements ExampleService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExampleServiceImpl.class);

    private static final HttpPoolClient httpPoolClient = HttpClientUtil.useDefault();

    /**
     * 线程池队列大小
     */
    private static final int QUEUE_MAX_SIZE = 100000;
    /**
     * 线程命名
     */
    private static ThreadFactory namedThreadFactory = new ThreadFactoryBuilder().setNameFormat("example-service-pool-%d").build();
    /**
     * 线程池
     */
    private static ExecutorService pool;

    @PostConstruct
    public void init() {
        //init thread pool
        pool = new ThreadPoolExecutor(32, 32,
                0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<>(QUEUE_MAX_SIZE), namedThreadFactory, new ThreadPoolExecutor.AbortPolicy());
    }

    @PreDestroy
    public void destroy() {
        //gracefully shutdown
        pool.shutdown();
    }

    @Override
    public ExampleVO greeting(String name) {
        LOGGER.info("name:{}", name);
        ExampleVO vo = new ExampleVO();
        vo.setGreeting("hello, " + name);
        //20路召回
        CountDownLatch countDownLatch = new CountDownLatch(20);
        List<Long> videoIds = new CopyOnWriteArrayList<>();
        //召回1
        pool.execute(() -> {
            try {
                //TODO
//                videoIds.addAll();
            } catch (Exception e) {
                LOGGER.error("recall 1 error", e);
            } finally {
                countDownLatch.countDown();
            }
        });
        //召回2
        pool.execute(() -> {
            try {
                //TODO
//                videoIds.addAll();
            } catch (Exception e) {
                LOGGER.error("recall 2 error", e);
            } finally {
                countDownLatch.countDown();
            }
        });
        //召回3
        pool.execute(() -> {
            try {
                //TODO
//                videoIds.addAll();
            } catch (Exception e) {
                LOGGER.error("recall 3 error", e);
            } finally {
                countDownLatch.countDown();
            }
        });

        //...

        try {
            countDownLatch.await(1000, TimeUnit.MICROSECONDS);
        } catch (InterruptedException e) {
            LOGGER.error("recall error", e);
        }

        return vo;
    }

}
